@@ -25,8 +25,6 @@
 #ifndef GKRDBUS_H_
 #define GKRDBUS_H_
 
-#ifdef WITH_DBUS 
-
 #include <glib.h>
 #include <dbus/dbus.h>
 
@@ -34,6 +32,4 @@ void gkr_dbus_connect_with_mainloop (DBusConnection *connection, GMainContext *c
 
 void gkr_dbus_disconnect_from_mainloop (DBusConnection *connection, GMainContext *context);
 
-#endif /* WITH_DBUS */
-
 #endif /*GKRDBUS_H_*/
