@@ -28,25 +28,16 @@
 #include "common/gkr-cleanup.h"
 #include "common/gkr-crypto.h"
 #include "common/gkr-daemon-util.h"
-#include "common/gkr-secure-memory.h"
+#include "egg/egg-secure-memory.h"
+#include "egg/egg-unix-credentials.h"
 #include "common/gkr-unix-signal.h"
 
 #include "keyrings/gkr-keyring-login.h"
 
 #include "library/gnome-keyring.h"
 
-#include "pk/gkr-pk-object-storage.h"
-#ifdef ROOT_CERTIFICATES
-#include "pk/gkr-pk-root-storage.h"
-#endif
-
 #include "pkcs11/gkr-pkcs11-daemon.h"
 
-#ifdef WITH_SSH
-#include "ssh/gkr-ssh-daemon.h"
-#include "ssh/gkr-ssh-storage.h"
-#endif
-
 #include "ui/gkr-ask-daemon.h"
 
 #include <unistd.h>
@@ -89,25 +80,30 @@ static GMainLoop *loop = NULL;
 
 /* All the components to run on startup if not set in gconf */
 #ifdef WITH_SSH
-#define DEFAULT_COMPONENTS  "ssh,keyring,pkcs11"
+#define DEFAULT_COMPONENTS  "ssh,pkcs11"
 #else
-#define DEFAULT_COMPONENTS  "keyring,pkcs11"
+#define DEFAULT_COMPONENTS  "pkcs11"
 #endif
 
 static gboolean run_foreground = FALSE;
 static gboolean run_daemonized = FALSE;
-static gboolean unlock_with_login = FALSE;
+static gboolean run_for_login = FALSE;
+static gboolean run_for_start = FALSE;
 static gchar* run_components = NULL;
+static gchar* login_password = NULL;
+static gboolean initialization_completed = FALSE;
 
 static GOptionEntry option_entries[] = {
 	{ "foreground", 'f', 0, G_OPTION_ARG_NONE, &run_foreground, 
 	  "Run in the foreground", NULL }, 
 	{ "daemonize", 'd', 0, G_OPTION_ARG_NONE, &run_daemonized, 
 	  "Run as a daemon", NULL }, 
-	{ "login", 'l', 0, G_OPTION_ARG_NONE, &unlock_with_login, 
-	  "Use login password from stdin", NULL },
+	{ "login", 'l', 0, G_OPTION_ARG_NONE, &run_for_login, 
+	  "Run for a user login. Read login password from stdin", NULL },
+	{ "start", 's', 0, G_OPTION_ARG_NONE, &run_for_start,
+	  "Start a dameon or initialize an already running daemon." },
 	{ "components", 'c', 0, G_OPTION_ARG_STRING, &run_components,
-	  "The components to run", DEFAULT_COMPONENTS },
+	  "The optional components to run", DEFAULT_COMPONENTS },
 	{ NULL }
 };
 
@@ -131,6 +127,13 @@ parse_arguments (int *argc, char** argv[])
 		gkr_cleanup_register (g_free, run_components);
 	}
 	
+	/* Check the arguments */
+	if (run_for_login && run_for_start) {
+		g_printerr ("gnome-keyring-daemon: The --start option is incompatible with --login");
+		run_for_login = FALSE;
+	}
+		
+	
 	g_option_context_free (context);
 }
 
@@ -217,19 +220,19 @@ static gboolean do_warning = TRUE;
  */ 
 
 void
-gkr_memory_lock (void)
+egg_memory_lock (void)
 {
 	/* The daemon uses cooperative threading, and doesn't need locking */
 }
 
 void 
-gkr_memory_unlock (void)
+egg_memory_unlock (void)
 {
 	/* The daemon uses cooperative threading, and doesn't need locking */
 }
 
 void*
-gkr_memory_fallback (void *p, unsigned long sz)
+egg_memory_fallback (void *p, unsigned long sz)
 {
 	const gchar *env;
 	
@@ -326,10 +329,16 @@ prepare_logging ()
     g_log_set_default_handler (log_handler, NULL);
 }
 
+void
+gkr_daemon_quit (void)
+{
+	g_main_loop_quit (loop);
+}
+
 static gboolean
 signal_handler (guint sig, gpointer unused)
 {
-	g_main_loop_quit (loop);
+	gkr_daemon_quit ();
 	return TRUE;
 }
 
@@ -352,8 +361,14 @@ read_login_password (int fd)
 	/* We only accept a max of 8K as the login password */
 	#define MAX_LENGTH 8192
 	#define MAX_BLOCK 256
+
+	/* 
+	 * When --login is specified then the login password is passed 
+	 * in on stdin. All data (including newlines) are part of the 
+	 * password.
+	 */
 	
-	gchar *buf = gkr_secure_alloc (MAX_BLOCK);
+	gchar *buf = egg_secure_alloc (MAX_BLOCK);
 	gchar *ret = NULL;
 	int r, len = 0;
 	
@@ -362,12 +377,12 @@ read_login_password (int fd)
 		if (r < 0) {
 			if (errno == EAGAIN)
 				continue;
-			gkr_secure_free (ret);
-			gkr_secure_free (buf);
+			egg_secure_free (ret);
+			egg_secure_free (buf);
 			return NULL;
 			
 		} else  { 
-			char *n = gkr_secure_realloc (ret, len + r + 1);
+			char *n = egg_secure_realloc (ret, len + r + 1);
 			memset(n + len, 0, r + 1); 
 			ret = n;
 			len = len + r;
@@ -379,33 +394,23 @@ read_login_password (int fd)
 			break;
 	}
 	
-	gkr_secure_free (buf);
+	egg_secure_free (buf);
 	return ret;
 }
 
-static void
-close_stdinout (void)
-{
-	int fd;
-	
-	fd = open ("/dev/null", O_RDONLY);
-	sane_dup2 (fd, 0);
-	close (fd);
-	
-	fd = open ("/dev/null", O_WRONLY);
-	sane_dup2 (fd, 1);
-	close (fd);
-
-	fd = open ("/dev/null", O_WRONLY);
-	sane_dup2 (fd, 2);
-	close (fd);
-}
-
 static void
 cleanup_and_exit (int code)
 {
 	gkr_cleanup_perform ();
-	_exit (code);
+	exit (code);
+}
+
+static void
+clear_login_password (void)
+{
+	if(login_password)
+		egg_secure_strfree (login_password);
+	login_password = NULL;
 }
 
 static gboolean
@@ -413,35 +418,264 @@ lifetime_slave_pipe_io (GIOChannel  *channel,
 			GIOCondition cond,
 			gpointer     callback_data)
 {
-	cleanup_and_exit (2);
+	gkr_cleanup_perform ();
+	_exit (2);
 	return FALSE;
 }
 
+static void
+slave_lifetime_to_fd (void)
+{
+	const char *env;
+	GIOChannel *channel;
+	int fd;
+	
+	env = getenv ("GNOME_KEYRING_LIFETIME_FD");
+	if (env && env[0]) {
+		fd = atoi (env);
+		if (fd != 0) {
+			channel = g_io_channel_unix_new (fd);
+			g_io_add_watch (channel,
+					G_IO_IN | G_IO_HUP,
+					lifetime_slave_pipe_io, NULL);
+			g_io_channel_unref (channel);
+		}
+	}
+}
+
 static void
 print_environment (pid_t pid)
 {
 	const gchar **env;
 	for (env = gkr_daemon_util_get_environment (); *env; ++env)
 		printf ("%s\n", *env);
-	printf ("GNOME_KEYRING_PID=%d\n", (gint)pid);
+	if (pid)
+		printf ("GNOME_KEYRING_PID=%d\n", (gint)pid);
+}
+
+static gboolean
+initialize_other_running_daemon (int sock)
+{
+	GnomeKeyringResult res;
+	gchar **envp, **e;
+	EggBuffer buf;
+	gboolean ret;
+	
+	if (egg_unix_credentials_write (sock) < 0)
+		return FALSE;
+
+	egg_buffer_init_full (&buf, 128, (EggBufferAllocator)g_realloc);
+	
+	envp = gnome_keyring_build_environment (GNOME_KEYRING_IN_ENVIRONMENT);
+	ret = gkr_proto_encode_prepare_environment (&buf, (const gchar**)envp);
+	g_strfreev (envp);
+	
+	if (!ret) {
+		egg_buffer_uninit (&buf);
+		g_return_val_if_reached (FALSE);
+	}
+
+	envp = NULL;
+
+	ret = gnome_keyring_socket_write_buffer (sock, &buf) && 
+	      gnome_keyring_socket_read_buffer (sock, &buf) && 
+	      gkr_proto_decode_prepare_environment_reply (&buf, &res, &envp);
+	
+	
+	egg_buffer_uninit (&buf);
+	
+	if(!ret) {
+		g_warning ("couldn't initialize running daemon");
+		return FALSE;
+	}
+
+	if (res == GNOME_KEYRING_RESULT_OK) {
+		g_return_val_if_fail (envp, FALSE);
+		for (e = envp; *e; ++e)
+			gkr_daemon_util_push_environment_full (*e);
+		ret = TRUE;
+	} else {
+		g_warning ("couldn't initialize running daemon: %s", gnome_keyring_result_to_message (res));
+		ret = FALSE;
+	}
+	
+	g_strfreev (envp);
+
+	return ret;
+}
+
+static gboolean
+start_or_initialize_daemon (void)
+{
+	gboolean ret;
+	int sock;
+	
+	/* 
+	 * Is a daemon already running? If not we need to run
+	 * a daemon process, just return and let things go 
+	 * their normal way. 
+	 */
+	sock = gnome_keyring_socket_connect_daemon (FALSE, TRUE);
+	if (sock == -1)
+		return FALSE;
+	
+	ret = initialize_other_running_daemon (sock);
+	close (sock);
+	
+	/* Initialization failed, start this process up as a daemon */
+	if (!ret)
+		return FALSE;
+	
+	/* 
+	 * Now we've initialized the daemon, we need to print out 
+	 * the daemon's environment for any callers, and possibly
+	 * block if we've been asked to remain in the foreground.
+	 */
+	print_environment (0);
+	
+	/* TODO: Better way to sleep forever? */
+	if (run_foreground) {
+		while (sleep(0x08000000) == 0);
+	}
+	
+	return TRUE;
+}
+
+static void
+fork_and_print_environment (void)
+{
+	int status;
+	pid_t pid;
+	int fd, i;
+
+	if (run_foreground) {
+		print_environment (getpid ());
+		return;
+	}
+	
+	pid = fork ();
+		
+	if (pid != 0) {
+
+		/* Here we are in the initial process */
+
+		if (run_daemonized) {
+			
+			/* Initial process, waits for intermediate child */
+			if (pid == -1)
+				exit (1);
+
+			waitpid (pid, &status, 0);
+			if (WEXITSTATUS (status) != 0)
+				exit (WEXITSTATUS (status));
+			
+		} else {
+			/* Not double forking, we know the PID */
+			print_environment (pid);
+		}
+
+		/* The initial process exits successfully */
+		exit (0);
+	}
+	
+	if (!run_daemonized) 
+		return;
+	
+	/* Double fork if need to daemonize properly */
+	pid = fork ();
+	
+	if (pid != 0) {
+
+		/* Here we are in the intermediate child process */
+			
+		/* 
+		 * This process exits, so that the final child will inherit 
+		 * init as parent to avoid zombies
+		 */
+		if (pid == -1)
+			exit (1);
+
+		/* We've done two forks. Now we know the PID */
+		print_environment (pid);
+			
+		/* The intermediate child exits */
+		exit (0);
+	}
+
+	/* Here we are in the resulting daemon process. */
+
+	for (i = 0; i < 3; ++i) {
+		fd = open ("/dev/null", O_RDONLY);
+		sane_dup2 (fd, i);
+		close (fd);
+	}
+}
+
+gboolean
+gkr_daemon_complete_initialization(void)
+{
+	/*
+	 * Sometimes we don't initialize the full daemon right on 
+	 * startup. When run with --login is one such case.
+	 */
+	
+	if (initialization_completed) {
+		g_message ("The daemon was already initialized.");
+		return TRUE;
+	}
+	
+	/* Initialize new style PKCS#11 components */
+	if (!gkr_pkcs11_daemon_initialize ())
+		return FALSE;
+	
+	/* Initialize the appropriate components */
+	
+#ifdef WITH_SSH	
+	if (check_run_component ("ssh")) {
+		if (!gkr_pkcs11_daemon_setup_ssh ())
+			return FALSE;
+	}
+#endif
+	
+	if (check_run_component ("pkcs11")) {
+		if (!gkr_pkcs11_daemon_setup_pkcs11 ())
+			return FALSE;
+	}
+	
+	gkr_daemon_dbus_setup ();
+	
+	initialization_completed = TRUE;
+	return TRUE;
 }
 
 int
 main (int argc, char *argv[])
 {
-	const char *env;
-	int fd;
-	pid_t pid;
-	GIOChannel *channel;
 	GMainContext *ctx;
-	gchar *login_password;
-	unsigned seed;
+	
+	/* 
+	 * The gnome-keyring startup is not as simple as I wish it could be. 
+	 * 
+	 * It's often started in the primidoral stages of a session, where 
+	 * there's no DBus, no GConf, and no proper X display. This is the 
+	 * strange world of PAM.
+	 * 
+	 * When started with the --login option, we do as little initialization
+	 * as possible. We expect a login password on the stdin, and unlock
+	 * or create the login keyring.
+	 * 
+	 * Then later we expect gnome-keyring-dameon to be run again with the 
+	 * --start option. This second gnome-keyring-daemon will hook the
+	 * original daemon up with environment variables necessary to initialize
+	 * itself and bring it into the session. This second daemon usually exits.
+	 * 
+	 * Without either of these options, we follow a more boring and 
+	 * predictable startup.  
+	 */
 	
 	g_type_init ();
 	g_thread_init (NULL);
 	
-	parse_arguments (&argc, &argv);
-
 #ifdef HAVE_LOCALE_H
 	/* internationalisation */
 	setlocale (LC_ALL, "");
@@ -455,153 +689,63 @@ main (int argc, char *argv[])
 
 	gkr_crypto_setup ();
 
-	gcry_create_nonce (&seed, sizeof (seed));
-	srand (seed);
+	/* Send all warning or error messages to syslog */
+	prepare_logging ();
 	
-	/* Initialize object storage */
-	if (!gkr_pk_object_storage_initialize ())
-		cleanup_and_exit (1);
+	parse_arguments (&argc, &argv);
 	
-#ifdef ROOT_CERTIFICATES
-	if (!gkr_pk_root_storage_initialize ())
-		cleanup_and_exit (1);
-#endif
-
-	/* Initialize the appropriate components */
-	if (check_run_component ("keyring")) {
-		if (!gkr_daemon_io_create_master_socket ())
-			cleanup_and_exit (1);
-	}
-
-#ifdef WITH_SSH	
-	if (check_run_component ("ssh")) {
-		if (!gkr_daemon_ssh_io_initialize () ||
-		    !gkr_ssh_storage_initialize ())
-			cleanup_and_exit (1);
-	}
-#endif
-	
-	if (check_run_component ("pkcs11")) {
-		if (!gkr_pkcs11_daemon_setup ())
-			cleanup_and_exit (1);
-	}	
-	 
-	/* 
-	 * When --login is specified then the login password is passed 
-	 * in on stdin. All data (including newlines) are part of the 
-	 * password.
-	 */
-	login_password = unlock_with_login ? read_login_password (STDIN) : NULL;
-	
-	/* 
-	 * The whole forking and daemonizing dance starts here.
-	 */
-	if (!run_foreground) {
-		pid = fork ();
-		
-		/* An intermediate child */
-		if (pid == 0) {
-			if (run_daemonized) {
-				pid = fork ();
-				
-				/* Still in the intermedate child */
-				if (pid != 0) {
-					gkr_secure_free (login_password);
-					
-					/* This process exits, so that the
-					 * final child will inherit init as parent
-					 * to avoid zombies
-					 */
-					if (pid == -1) {
-						exit (1);
-					} else {
-						/* This is where we know the pid of the daemon.
-						 * The initial process will waitpid until we exit,
-						 * so there is no race */
-						print_environment (pid);
-						exit (0);
-					}
-				}
-			}
-			
-			/* final child continues here */
-			
-		/* The initial process */
-		} else {
-			gkr_secure_free (login_password);
-			
-			if (run_daemonized) {
-				int status;
-				
-				/* Initial process, waits for intermediate child */
-				if (pid == -1)
-					exit (1);
-
-				waitpid (pid, &status, 0);
-				if (WEXITSTATUS (status) != 0)
-					exit (WEXITSTATUS (status));
-				
-			} else {
-				print_environment (pid);
-			}
-			
-			exit (0);
-		}
-		
-		/* The final child ... */
-		close_stdinout ();
-
-	} else {
-		print_environment (getpid ());
-	}
-
-	/* Daemon process continues here */
-
-        /* Send all warning or error messages to syslog */
-	prepare_logging();
+	/* The --start option */
+	if (run_for_start) {
+		if (start_or_initialize_daemon ())
+			cleanup_and_exit (0);
+	} 
 
+	/* Initialize our daemon main loop and threading */
 	loop = g_main_loop_new (NULL, FALSE);
 	ctx = g_main_loop_get_context (loop);
+	gkr_async_workers_init (loop);
 	
+	/* 
+	 * Always initialize the keyring subsystem. This is a necessary
+	 * component that everything else depends on in one way or 
+	 * another. 
+	 */
+	if (!gkr_daemon_io_create_master_socket ())
+		cleanup_and_exit (1);
+
+	/* The --login option. Delayed initialization */
+	if (run_for_login) {
+		login_password = read_login_password (STDIN);
+		atexit (clear_login_password);
+	
+	/* Not a login daemon. Initialize now. */
+	} else {
+		if (!gkr_daemon_complete_initialization ())
+			cleanup_and_exit (1);
+	}
+	 
+	/* The whole forking and daemonizing dance starts here. */
+	fork_and_print_environment();
+
+	/* Prepare logging a second time, since we may be in a different process */
+	prepare_logging();
+
 	signal (SIGPIPE, SIG_IGN);
 	gkr_unix_signal_connect (ctx, SIGINT, signal_handler, NULL);
 	gkr_unix_signal_connect (ctx, SIGHUP, signal_handler, NULL);
 	gkr_unix_signal_connect (ctx, SIGTERM, signal_handler, NULL);
              
-	env = getenv ("GNOME_KEYRING_LIFETIME_FD");
-	if (env && env[0]) {
-		fd = atoi (env);
-		if (fd != 0) {
-			channel = g_io_channel_unix_new (fd);
-			g_io_add_watch (channel,
-					G_IO_IN | G_IO_HUP,
-					lifetime_slave_pipe_io, NULL);
-			g_io_channel_unref (channel);
-		}
-		
-	}
-	
-	gkr_async_workers_init (loop);
-	
-#ifdef WITH_DBUS
-	/* 
-	 * We may be launched before the DBUS session, (ie: via PAM) 
-	 * and DBus tries to launch itself somehow, so double check 
-	 * that it has really started.
-	 */ 
-	env = getenv ("DBUS_SESSION_BUS_ADDRESS");
-	if (env && env[0])
-		gkr_daemon_dbus_setup (loop);
-#endif
+	/* TODO: Do we still need this? XFCE still seems to use it. */
+	slave_lifetime_to_fd ();
 
 	/*
 	 * Unlock the login keyring if we were given a password on STDIN.
 	 * If it does not exist. We create it. 
 	 */
-	if (unlock_with_login && login_password) {
+	if (login_password) {
 		if (!gkr_keyring_login_unlock (login_password))
 			g_warning ("Failed to unlock login on startup");
-		gkr_secure_free (login_password);
+		egg_secure_strclear (login_password);
 	}
 	
 	g_main_loop_run (loop);
