@@ -200,7 +200,7 @@ create_rsa_private (GArray *attrs, gcry_sexp_t *skey)
 	
 	gkr_pk_attributes_consume (attrs, CKA_MODULUS, CKA_PUBLIC_EXPONENT, 
 	                           CKA_PRIVATE_EXPONENT, CKA_PRIME_1, CKA_PRIME_2, 
-	                           CKA_EXPONENT_1, CKA_EXPONENT_2, CKA_COEFFICIENT, -1);
+	                           CKA_EXPONENT_1, CKA_EXPONENT_2, CKA_COEFFICIENT, G_MAXULONG);
 	ret = CKR_OK;
 
 done:
@@ -249,7 +249,7 @@ create_dsa_private (GArray *attrs, gcry_sexp_t *skey)
 	}
 
 	gkr_pk_attributes_consume (attrs, CKA_PRIME, CKA_SUBPRIME, 
-	                           CKA_BASE, CKA_VALUE, -1);
+	                           CKA_BASE, CKA_VALUE, G_MAXULONG);
 	ret = CKR_OK;
 
 done:
@@ -642,7 +642,7 @@ gkr_pk_privkey_create (GkrPkManager* manager, GArray* array,
 	
 	if (!gkr_pk_attributes_ulong (array, CKA_KEY_TYPE, &type))
  		return CKR_TEMPLATE_INCOMPLETE;
- 	gkr_pk_attributes_consume (array, CKA_KEY_TYPE, -1);
+ 	gkr_pk_attributes_consume (array, CKA_KEY_TYPE, G_MAXULONG);
 
  	switch (type) {
 	case CKK_RSA:
