@@ -28,6 +28,9 @@
 #include "gck-data-der.h"
 #include "gck-data-types.h"
 
+#include "egg/egg-secure-memory.h"
+#include "egg/egg-symkey.h"
+
 #include <glib.h>
 #include <gcrypt.h>
 #include <libtasn1.h>
@@ -38,27 +41,7 @@
 
 static GQuark OID_PKIX1_RSA;
 static GQuark OID_PKIX1_DSA;
-
-static GQuark OID_PBE_MD2_DES_CBC;
-static GQuark OID_PBE_MD5_DES_CBC;
-static GQuark OID_PBE_MD2_RC2_CBC;
-static GQuark OID_PBE_MD5_RC2_CBC;
-static GQuark OID_PBE_SHA1_DES_CBC;
-static GQuark OID_PBE_SHA1_RC2_CBC;
-static GQuark OID_PBES2;
-static GQuark OID_PBKDF2;
-
-static GQuark OID_DES_CBC;
-static GQuark OID_DES_RC2_CBC;
-static GQuark OID_DES_EDE3_CBC;
-static GQuark OID_DES_RC5_CBC;
-
-static GQuark OID_PKCS12_PBE_ARCFOUR_SHA1;
-static GQuark OID_PKCS12_PBE_RC4_40_SHA1;
 static GQuark OID_PKCS12_PBE_3DES_SHA1;
-static GQuark OID_PKCS12_PBE_2DES_SHA1;
-static GQuark OID_PKCS12_PBE_RC2_128_SHA1;
-static GQuark OID_PKCS12_PBE_RC2_40_SHA1;
 
 static void
 init_quarks (void)
@@ -72,29 +55,7 @@ init_quarks (void)
 
 		QUARK (OID_PKIX1_RSA, "1.2.840.113549.1.1.1");
 		QUARK (OID_PKIX1_DSA, "1.2.840.10040.4.1");
-
-		QUARK (OID_PBE_MD2_DES_CBC, "1.2.840.113549.1.5.1");
-		QUARK (OID_PBE_MD5_DES_CBC, "1.2.840.113549.1.5.3");
-		QUARK (OID_PBE_MD2_RC2_CBC, "1.2.840.113549.1.5.4");
-		QUARK (OID_PBE_MD5_RC2_CBC, "1.2.840.113549.1.5.6");
-		QUARK (OID_PBE_SHA1_DES_CBC, "1.2.840.113549.1.5.10");
-		QUARK (OID_PBE_SHA1_RC2_CBC, "1.2.840.113549.1.5.11");
-		
-		QUARK (OID_PBES2, "1.2.840.113549.1.5.13");
-		
-		QUARK (OID_PBKDF2, "1.2.840.113549.1.5.12");
-		
-		QUARK (OID_DES_CBC, "1.3.14.3.2.7");
-		QUARK (OID_DES_RC2_CBC, "1.2.840.113549.3.2");
-		QUARK (OID_DES_EDE3_CBC, "1.2.840.113549.3.7");
-		QUARK (OID_DES_RC5_CBC, "1.2.840.113549.3.9");
-		
-		QUARK (OID_PKCS12_PBE_ARCFOUR_SHA1, "1.2.840.113549.1.12.1.1");
-		QUARK (OID_PKCS12_PBE_RC4_40_SHA1, "1.2.840.113549.1.12.1.2");
 		QUARK (OID_PKCS12_PBE_3DES_SHA1, "1.2.840.113549.1.12.1.3");
-		QUARK (OID_PKCS12_PBE_2DES_SHA1, "1.2.840.113549.1.12.1.4");
-		QUARK (OID_PKCS12_PBE_RC2_128_SHA1, "1.2.840.113549.1.12.1.5");
-		QUARK (OID_PKCS12_PBE_RC2_40_SHA1, "1.2.840.113549.1.12.1.6");
 		
 		#undef QUARK
 		
@@ -122,7 +83,7 @@ gck_data_der_read_public_key_rsa (const guchar *data, gsize n_data, gcry_sexp_t
 
 	n = e = NULL;
 	
-	asn = gck_data_asn1_decode ("PK.RSAPublicKey", data, n_data);
+	asn = egg_asn1_decode ("PK.RSAPublicKey", data, n_data);
 	if (!asn)
 		goto done;
 		
@@ -173,13 +134,13 @@ gck_data_der_read_private_key_rsa (const guchar *data, gsize n_data, gcry_sexp_t
 
 	n = e = d = p = q = u = NULL;
 	
-	asn = gck_data_asn1_decode ("PK.RSAPrivateKey", data, n_data);
+	asn = egg_asn1_decode ("PK.RSAPrivateKey", data, n_data);
 	if (!asn)
 		goto done;
 		
 	ret = GCK_DATA_FAILURE;
 	
-	if (!gck_data_asn1_read_uint (asn, "version", &version))
+	if (!egg_asn1_read_uint (asn, "version", &version))
 		goto done;
 	
 	/* We only support simple version */
@@ -249,7 +210,7 @@ gck_data_der_read_public_key_dsa (const guchar *data, gsize n_data, gcry_sexp_t
 
 	p = q = g = y = NULL;
 	
-	asn = gck_data_asn1_decode ("PK.DSAPublicKey", data, n_data);
+	asn = egg_asn1_decode ("PK.DSAPublicKey", data, n_data);
 	if (!asn)
 		goto done;
 	
@@ -295,8 +256,8 @@ gck_data_der_read_public_key_dsa_parts (const guchar *keydata, gsize n_keydata,
 
 	p = q = g = y = NULL;
 	
-	asn_params = gck_data_asn1_decode ("PK.DSAParameters", params, n_params);
-	asn_key = gck_data_asn1_decode ("PK.DSAPublicPart", keydata, n_keydata);
+	asn_params = egg_asn1_decode ("PK.DSAParameters", params, n_params);
+	asn_key = egg_asn1_decode ("PK.DSAPublicPart", keydata, n_keydata);
 	if (!asn_params || !asn_key)
 		goto done;
 	
@@ -352,7 +313,7 @@ gck_data_der_read_private_key_dsa (const guchar *data, gsize n_data, gcry_sexp_t
 
 	p = q = g = y = x = NULL;
 	
-	asn = gck_data_asn1_decode ("PK.DSAPrivateKey", data, n_data);
+	asn = egg_asn1_decode ("PK.DSAPrivateKey", data, n_data);
 	if (!asn)
 		goto done;
 	
@@ -400,8 +361,8 @@ gck_data_der_read_private_key_dsa_parts (const guchar *keydata, gsize n_keydata,
 
 	p = q = g = y = x = NULL;
 	
-	asn_params = gck_data_asn1_decode ("PK.DSAParameters", params, n_params);
-	asn_key = gck_data_asn1_decode ("PK.DSAPrivatePart", keydata, n_keydata);
+	asn_params = egg_asn1_decode ("PK.DSAParameters", params, n_params);
+	asn_key = egg_asn1_decode ("PK.DSAPrivatePart", keydata, n_keydata);
 	if (!asn_params || !asn_key)
 		goto done;
 	
@@ -467,19 +428,19 @@ gck_data_der_read_public_key_info (const guchar* data, gsize n_data, gcry_sexp_t
 	
 	init_quarks ();
 
-	asn = gck_data_asn1_decode ("PKIX1.SubjectPublicKeyInfo", data, n_data);
+	asn = egg_asn1_decode ("PKIX1.SubjectPublicKeyInfo", data, n_data);
 	if (!asn)
 		goto done;
 	
 	ret = GCK_DATA_FAILURE;
     
 	/* Figure out the algorithm */
-	oid = gck_data_asn1_read_oid (asn, "algorithm.algorithm");
+	oid = egg_asn1_read_oid (asn, "algorithm.algorithm");
 	if (!oid)
 		goto done;
 		
 	/* A bit string so we cannot process in place */
-	key = gck_data_asn1_read_value (asn, "subjectPublicKey", &n_key, NULL);
+	key = egg_asn1_read_value (asn, "subjectPublicKey", &n_key, NULL);
 	if (!key)
 		goto done;
 	n_key /= 8;
@@ -490,7 +451,7 @@ gck_data_der_read_public_key_info (const guchar* data, gsize n_data, gcry_sexp_t
 		
 	/* A DSA key paramaters are stored separately */
 	} else if (oid == OID_PKIX1_DSA) {
-		params = gck_data_asn1_read_element (asn, data, n_data, "algorithm.parameters", &n_params);
+		params = egg_asn1_read_element (asn, data, n_data, "algorithm.parameters", &n_params);
 		if (!params)
 			goto done;
 		ret = gck_data_der_read_public_key_dsa_parts (key, n_key, params, n_params, s_key);
@@ -524,6 +485,180 @@ gck_data_der_read_private_key (const guchar *data, gsize n_data, gcry_sexp_t *s_
 	return res;
 }
 
+GckDataResult
+gck_data_der_read_private_pkcs8_plain (const guchar *data, gsize n_data, gcry_sexp_t *s_key)
+{
+	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
+	GckDataResult ret;
+	int algorithm;
+	GQuark key_algo;
+	const guchar *keydata;
+	gsize n_keydata;
+	const guchar *params;
+	gsize n_params;
+	
+	ret = GCK_DATA_UNRECOGNIZED;
+	
+	init_quarks ();
+	
+	asn = egg_asn1_decode ("PKIX1.pkcs-8-PrivateKeyInfo", data, n_data);
+	if (!asn)
+		goto done;
+
+	ret = GCK_DATA_FAILURE;
+	algorithm = 0;
+		
+	key_algo = egg_asn1_read_oid (asn, "privateKeyAlgorithm.algorithm");
+  	if (!key_algo)
+  		goto done;
+  	else if (key_algo == OID_PKIX1_RSA)
+  		algorithm = GCRY_PK_RSA;
+  	else if (key_algo == OID_PKIX1_DSA)
+  		algorithm = GCRY_PK_DSA;
+  		
+  	if (!algorithm) {
+  		ret = GCK_DATA_UNRECOGNIZED;
+  		goto done;
+  	}
+
+	keydata = egg_asn1_read_content (asn, data, n_data, "privateKey", &n_keydata);
+	if (!keydata)
+		goto done;
+		
+	params = egg_asn1_read_element (asn, data, n_data, "privateKeyAlgorithm.parameters", 
+	                                     &n_params);
+		
+	ret = GCK_DATA_SUCCESS;
+	
+done:
+	if (ret == GCK_DATA_SUCCESS) {		
+		switch (algorithm) {
+		case GCRY_PK_RSA:
+			ret = gck_data_der_read_private_key_rsa (keydata, n_keydata, s_key);
+			break;
+		case GCRY_PK_DSA:
+			/* Try the normal one block format */
+			ret = gck_data_der_read_private_key_dsa (keydata, n_keydata, s_key);
+			
+			/* Otherwise try the two part format that everyone seems to like */
+			if (ret == GCK_DATA_UNRECOGNIZED && params && n_params)
+				ret = gck_data_der_read_private_key_dsa_parts (keydata, n_keydata, 
+				                                               params, n_params, s_key);
+			break;
+		default:
+			g_message ("invalid or unsupported key type in PKCS#8 key");
+			ret = GCK_DATA_UNRECOGNIZED;
+			break;
+		};
+		
+	} else if (ret == GCK_DATA_FAILURE) {
+		g_message ("invalid PKCS#8 key");
+	}
+	
+	if (asn)
+		asn1_delete_structure (&asn);
+	
+	return ret;
+}
+
+GckDataResult
+gck_data_der_read_private_pkcs8_crypted (const guchar *data, gsize n_data, const gchar *password, 
+                                         gsize n_password, gcry_sexp_t *s_key)
+{
+	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
+	gcry_cipher_hd_t cih = NULL;
+	gcry_error_t gcry;
+	GckDataResult ret, r;
+	GQuark scheme;
+	guchar *crypted = NULL;
+	const guchar *params;
+	gsize n_crypted, n_params;
+	gint l;
+
+	init_quarks ();
+
+	ret = GCK_DATA_UNRECOGNIZED;
+	
+	asn = egg_asn1_decode ("PKIX1.pkcs-8-EncryptedPrivateKeyInfo", data, n_data);
+	if (!asn)
+		goto done;
+
+	ret = GCK_DATA_FAILURE;
+
+	/* Figure out the type of encryption */
+	scheme = egg_asn1_read_oid (asn, "encryptionAlgorithm.algorithm");
+	if (!scheme)
+		goto done;
+		
+	params = egg_asn1_read_element (asn, data, n_data, "encryptionAlgorithm.parameters", &n_params);
+	if (!params)
+		goto done;
+
+	/* 
+	 * Parse the encryption stuff into a cipher. 
+	 */
+	r = egg_symkey_read_cipher (scheme, password, n_password, params, n_params, &cih);
+	if (r == GCK_DATA_UNRECOGNIZED) {
+		ret = GCK_DATA_FAILURE;
+		goto done;
+	} else if (r != GCK_DATA_SUCCESS) {
+		ret = r;
+		goto done;
+	}
+			
+	crypted = egg_asn1_read_value (asn, "encryptedData", &n_crypted, (EggAllocator)egg_secure_realloc);
+	if (!crypted)
+		goto done;
+	
+	gcry = gcry_cipher_decrypt (cih, crypted, n_crypted, NULL, 0);
+	gcry_cipher_close (cih);
+	cih = NULL;
+		
+	if (gcry != 0) {
+		g_warning ("couldn't decrypt pkcs8 data: %s", gcry_strerror (gcry));
+		goto done;
+	}
+		
+	/* Unpad the DER data */
+	l = egg_asn1_element_length (crypted, n_crypted);
+	if (l <= 0 || l > n_crypted) {
+		ret = GCK_DATA_LOCKED;
+		goto done;
+	} 
+	n_crypted = l;
+		
+	/* Try to parse the resulting key */
+	ret = gck_data_der_read_private_pkcs8_plain (crypted, n_crypted, s_key);
+	egg_secure_free (crypted);
+	crypted = NULL;
+		
+	/* If unrecognized we assume bad password */
+	if (ret == GCK_DATA_UNRECOGNIZED) 
+		ret = GCK_DATA_LOCKED;
+
+done:
+	if (cih)
+		gcry_cipher_close (cih);
+	if (asn)
+		asn1_delete_structure (&asn);
+	egg_secure_free (crypted);
+		
+	return ret;
+}
+
+GckDataResult
+gck_data_der_read_private_pkcs8 (const guchar *data, gsize n_data, const gchar *password, 
+                                 gsize n_password, gcry_sexp_t *s_key)
+{
+	GckDataResult res;
+
+	res = gck_data_der_read_private_pkcs8_crypted (data, n_data, password, n_password, s_key);
+	if (res == GCK_DATA_UNRECOGNIZED) 
+		res = gck_data_der_read_private_pkcs8_plain (data, n_data, s_key);
+
+	return res;
+}
+
 guchar*
 gck_data_der_write_public_key_rsa (gcry_sexp_t s_key, gsize *len)
 {
@@ -534,7 +669,7 @@ gck_data_der_write_public_key_rsa (gcry_sexp_t s_key, gsize *len)
 
 	n = e = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.RSAPublicKey", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -546,7 +681,7 @@ gck_data_der_write_public_key_rsa (gcry_sexp_t s_key, gsize *len)
 	    !gck_data_asn1_write_mpi (asn, "publicExponent", e))
 	    	goto done;
 
-	result = gck_data_asn1_encode (asn, "", len, NULL);
+	result = egg_asn1_encode (asn, "", len, NULL);
 	
 done:
 	if (asn)
@@ -567,7 +702,7 @@ gck_data_der_write_private_key_rsa (gcry_sexp_t s_key, gsize *n_key)
 
 	n = e = d = p = q = u = e1 = e2 = tmp = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.RSAPrivateKey", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -602,10 +737,10 @@ gck_data_der_write_private_key_rsa (gcry_sexp_t s_key, gsize *n_key)
 		goto done;
 
 	/* Write out the version */
-	if (!gck_data_asn1_write_uint (asn, "version", 0))
+	if (!egg_asn1_write_uint (asn, "version", 0))
 		goto done;
 
-	result = gck_data_asn1_encode (asn, "", n_key, NULL);
+	result = egg_asn1_encode (asn, "", n_key, NULL);
 	
 done:
 	if (asn)
@@ -634,7 +769,7 @@ gck_data_der_write_public_key_dsa (gcry_sexp_t s_key, gsize *len)
 
 	p = q = g = y = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.DSAPublicKey", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -650,10 +785,10 @@ gck_data_der_write_public_key_dsa (gcry_sexp_t s_key, gsize *len)
 	    !gck_data_asn1_write_mpi (asn, "Y", y))
 	    	goto done;
 
-	if (!gck_data_asn1_write_uint (asn, "version", 0))
+	if (!egg_asn1_write_uint (asn, "version", 0))
 		goto done; 
 		
-	result = gck_data_asn1_encode (asn, "", len, NULL);
+	result = egg_asn1_encode (asn, "", len, NULL);
 	
 done:
 	if (asn)
@@ -676,7 +811,7 @@ gck_data_der_write_private_key_dsa_part (gcry_sexp_t skey, gsize *n_key)
 
 	x = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.DSAPrivatePart", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -686,7 +821,7 @@ gck_data_der_write_private_key_dsa_part (gcry_sexp_t skey, gsize *n_key)
 	if (!gck_data_asn1_write_mpi (asn, "", x))
 	    	goto done;
 
-	result = gck_data_asn1_encode (asn, "", n_key, NULL);
+	result = egg_asn1_encode (asn, "", n_key, NULL);
 	
 done:
 	if (asn)
@@ -706,7 +841,7 @@ gck_data_der_write_private_key_dsa_params (gcry_sexp_t skey, gsize *n_params)
 
 	p = q = g = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.DSAParameters", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -720,7 +855,7 @@ gck_data_der_write_private_key_dsa_params (gcry_sexp_t skey, gsize *n_params)
 	    !gck_data_asn1_write_mpi (asn, "g", g))
 	    	goto done;
 
-	result = gck_data_asn1_encode (asn, "", n_params, NULL);
+	result = egg_asn1_encode (asn, "", n_params, NULL);
 	
 done:
 	if (asn)
@@ -742,7 +877,7 @@ gck_data_der_write_private_key_dsa (gcry_sexp_t s_key, gsize *len)
 
 	p = q = g = y = x = NULL;
 
-	res = asn1_create_element (gck_data_asn1_get_pk_asn1type (), 
+	res = asn1_create_element (egg_asn1_get_pk_asn1type (), 
 	                           "PK.DSAPrivateKey", &asn);
 	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
 
@@ -760,10 +895,10 @@ gck_data_der_write_private_key_dsa (gcry_sexp_t s_key, gsize *len)
 	    !gck_data_asn1_write_mpi (asn, "priv", x))
 	    	goto done;
 
-	if (!gck_data_asn1_write_uint (asn, "version", 0))
+	if (!egg_asn1_write_uint (asn, "version", 0))
 		goto done; 
 		
-	result = gck_data_asn1_encode (asn, "", len, NULL);
+	result = egg_asn1_encode (asn, "", len, NULL);
 	
 done:
 	if (asn)
@@ -823,6 +958,199 @@ gck_data_der_write_private_key (gcry_sexp_t s_key, gsize *len)
 	}
 }
 
+static gcry_cipher_hd_t
+prepare_and_encode_pkcs8_cipher (ASN1_TYPE asn, const gchar *password, 
+                                 gsize n_password, gsize *n_block)
+{
+	ASN1_TYPE asn1_params;
+	gcry_cipher_hd_t cih;
+	guchar salt[8];
+	gcry_error_t gcry;
+	guchar *key, *iv, *portion;
+	gsize n_key, n_portion;
+	int iterations, res;
+	
+	init_quarks ();
+
+	/* Make sure the encryption algorithm works */
+	g_return_val_if_fail (gcry_cipher_algo_info (OID_PKCS12_PBE_3DES_SHA1, 
+	                                             GCRYCTL_TEST_ALGO, NULL, 0), NULL);
+
+	/* The encryption algorithm */
+	if(!egg_asn1_write_oid (asn, "encryptionAlgorithm.algorithm", 
+	                             OID_PKCS12_PBE_3DES_SHA1))
+		g_return_val_if_reached (NULL); 
+
+	/* Randomize some input for the password based secret */
+	iterations = 1000 + (int) (1000.0 * rand () / (RAND_MAX + 1.0));
+	gcry_create_nonce (salt, sizeof (salt));
+
+	/* Allocate space for the key and iv */
+	n_key = gcry_cipher_get_algo_keylen (GCRY_CIPHER_3DES);
+	*n_block = gcry_cipher_get_algo_blklen (GCRY_MD_SHA1);
+	g_return_val_if_fail (n_key && *n_block, NULL);
+		
+	if (!egg_symkey_generate_pkcs12 (GCRY_CIPHER_3DES, GCRY_MD_SHA1, 
+	                                        password, n_password, salt, 
+	                                        sizeof (salt), iterations, &key, &iv))
+		g_return_val_if_reached (NULL);
+
+	/* Now write out the parameters */	
+	res = asn1_create_element (egg_asn1_get_pkix_asn1type (),
+	                           "PKIX1.pkcs-12-PbeParams", &asn1_params);
+	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
+	if (!egg_asn1_write_value (asn1_params, "salt", salt, sizeof (salt)))
+		g_return_val_if_reached (NULL);
+	if (!egg_asn1_write_uint (asn1_params, "iterations", iterations))
+		g_return_val_if_reached (NULL);
+	portion = egg_asn1_encode (asn1_params, "", &n_portion, NULL);
+	g_return_val_if_fail (portion, NULL); 
+	
+	if (!egg_asn1_write_value (asn, "encryptionAlgorithm.parameters", portion, n_portion))
+		g_return_val_if_reached (NULL);
+	g_free (portion);
+	
+	/* Now make a cipher that matches what we wrote out */
+	gcry = gcry_cipher_open (&cih, GCRY_CIPHER_3DES, GCRY_CIPHER_MODE_CBC, 0);
+	g_return_val_if_fail (gcry == 0, NULL);
+	g_return_val_if_fail (cih, NULL);
+	
+	gcry_cipher_setiv (cih, iv, *n_block);
+	gcry_cipher_setkey (cih, key, n_key);
+	
+	g_free (iv);
+	gcry_free (key);
+	asn1_delete_structure (&asn1_params);
+	
+	return cih;
+}
+
+guchar*
+gck_data_der_write_private_pkcs8_plain (gcry_sexp_t skey, gsize *n_data)
+{
+	ASN1_TYPE asn;
+	int res, algorithm;
+	gboolean is_priv;
+	GQuark oid;
+	guchar *params, *key, *data;
+	gsize n_params, n_key;
+	
+	init_quarks ();
+
+	/* Parse and check that the key is for real */
+	if (!gck_crypto_sexp_parse_key (skey, &algorithm, &is_priv, NULL))
+		g_return_val_if_reached (NULL);
+	g_return_val_if_fail (is_priv == TRUE, NULL);
+	
+	res = asn1_create_element (egg_asn1_get_pkix_asn1type (), 
+	                           "PKIX1.pkcs-8-PrivateKeyInfo", &asn);
+	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
+	
+	/* Write out the version */
+	if (!egg_asn1_write_uint (asn, "version", 1))
+		g_return_val_if_reached (NULL);
+	
+	/* Per algorithm differences */
+	switch (algorithm)
+	{
+	/* RSA gets encoded in a standard simple way */
+	case GCRY_PK_RSA:
+		oid = OID_PKIX1_RSA;
+		params = NULL;
+		n_params = 0;
+		key = gck_data_der_write_private_key_rsa (skey, &n_key);
+		break;
+		
+	/* DSA gets incoded with the params seperate */
+	case GCRY_PK_DSA:
+		oid = OID_PKIX1_DSA;
+		key = gck_data_der_write_private_key_dsa_part (skey, &n_key);
+		params = gck_data_der_write_private_key_dsa_params (skey, &n_params);
+		break;
+		
+	default:
+		g_warning ("trying to serialize unsupported private key algorithm: %d", algorithm);
+		return NULL;
+	};
+	
+	/* Write out the algorithm */
+	if (!egg_asn1_write_oid (asn, "privateKeyAlgorithm.algorithm", oid))
+		g_return_val_if_reached (NULL);
+
+	/* Write out the parameters */
+	if (!egg_asn1_write_value (asn, "privateKeyAlgorithm.parameters", params, n_params))
+		g_return_val_if_reached (NULL);
+	egg_secure_free (params);
+	
+	/* Write out the key portion */
+	if (!egg_asn1_write_value (asn, "privateKey", key, n_key))
+		g_return_val_if_reached (NULL);
+	egg_secure_free (key);
+	
+	/* Add an empty attributes field */
+	if (!egg_asn1_write_value (asn, "attributes", NULL, 0))
+		g_return_val_if_reached (NULL);
+	
+	data = egg_asn1_encode (asn, "", n_data, NULL);
+	g_return_val_if_fail (data, NULL); 
+	
+	asn1_delete_structure (&asn);
+	
+	return data;
+}
+
+guchar*
+gck_data_der_write_private_pkcs8_crypted (gcry_sexp_t skey, const gchar *password,
+                                          gsize n_password, gsize *n_data)
+{
+	gcry_error_t gcry;
+	gcry_cipher_hd_t cih;
+	ASN1_TYPE asn;
+	int res;
+	guchar *key, *data; 
+	gsize n_key, block = 0;
+
+	/* Encode the key in normal pkcs8 fashion */
+	key = gck_data_der_write_private_pkcs8_plain (skey, &n_key);
+	
+	res = asn1_create_element (egg_asn1_get_pkix_asn1type (), 
+	                           "PKIX1.pkcs-8-EncryptedPrivateKeyInfo", &asn);
+	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
+	
+	/* Create a and write out a cipher used for encryption */
+	cih = prepare_and_encode_pkcs8_cipher (asn, password, n_password, &block);
+	g_return_val_if_fail (cih, NULL);
+	
+	/* Pad the block of data */
+	if(block > 1) {
+		gsize pad;
+		guchar *padded;
+		
+		pad = block - (n_key % block);
+		if (pad == 0)
+			pad = block;
+		padded = g_realloc (key, n_key + pad);
+		memset (padded + n_key, pad, pad);
+		key = padded;
+		n_key += pad;
+	}
+	
+	gcry = gcry_cipher_encrypt (cih, key, n_key, NULL, 0);
+	g_return_val_if_fail (gcry == 0, NULL);
+	
+	gcry_cipher_close (cih);
+	
+	res = asn1_write_value (asn, "encryptedData", key, n_key);
+	g_return_val_if_fail (res == ASN1_SUCCESS, NULL);
+	
+	data = egg_asn1_encode (asn, "", n_data, NULL);
+	g_return_val_if_fail (data, NULL); 
+
+	asn1_delete_structure (&asn);
+	
+	return data;
+}
+
 /* -----------------------------------------------------------------------------
  * CERTIFICATES
  */
@@ -830,7 +1158,7 @@ gck_data_der_write_private_key (gcry_sexp_t s_key, gsize *len)
 GckDataResult
 gck_data_der_read_certificate (const guchar *data, gsize n_data, ASN1_TYPE *asn1)
 {
-	*asn1 = gck_data_asn1_decode ("PKIX1.Certificate", data, n_data);
+	*asn1 = egg_asn1_decode ("PKIX1.Certificate", data, n_data);
 	if (!*asn1)
 		return GCK_DATA_UNRECOGNIZED;
 	
@@ -845,21 +1173,21 @@ gck_data_der_read_basic_constraints (const guchar *data, gsize n_data,
 	ASN1_TYPE asn;
 	guint value;
 
-	asn = gck_data_asn1_decode ("PKIX1.BasicConstraints", data, n_data);
+	asn = egg_asn1_decode ("PKIX1.BasicConstraints", data, n_data);
 	if (!asn)
 		goto done;
 	
 	ret = GCK_DATA_FAILURE;
     
     	if (path_len) {
-    		if (!gck_data_asn1_read_uint (asn, "pathLenConstraint", &value))
+    		if (!egg_asn1_read_uint (asn, "pathLenConstraint", &value))
     			*path_len = -1;
     		else
     			*path_len = value;
     	}
     	
     	if (is_ca) {
-    		if (!gck_data_asn1_read_boolean (asn, "cA", is_ca))
+    		if (!egg_asn1_read_boolean (asn, "cA", is_ca))
     			*is_ca = FALSE;
     	}
     	
@@ -885,7 +1213,7 @@ gck_data_der_read_key_usage (const guchar *data, gsize n_data, guint *key_usage)
 	guchar buf[4];
 	int res, len;
 	
-	asn = gck_data_asn1_decode ("PKIX1.KeyUsage", data, n_data);
+	asn = egg_asn1_decode ("PKIX1.KeyUsage", data, n_data);
 	if (!asn)
 		goto done;
 		
@@ -916,7 +1244,7 @@ gck_data_der_read_enhanced_usage (const guchar *data, gsize n_data, GQuark **usa
 	GQuark oid;
 	int i;
 	
-	asn = gck_data_asn1_decode ("PKIX1.ExtKeyUsageSyntax", data, n_data);
+	asn = egg_asn1_decode ("PKIX1.ExtKeyUsageSyntax", data, n_data);
 	if (!asn)
 		goto done;
 		
@@ -925,7 +1253,7 @@ gck_data_der_read_enhanced_usage (const guchar *data, gsize n_data, GQuark **usa
 	array = g_array_new (TRUE, TRUE, sizeof (GQuark));
 	for (i = 0; TRUE; ++i) {
 		part = g_strdup_printf ("?%d", i + 1);
-		oid = gck_data_asn1_read_oid (asn, part);
+		oid = egg_asn1_read_oid (asn, part);
 		g_free (part);
 		
 		if (!oid) 
@@ -943,446 +1271,13 @@ done:
 	return ret;
 }
 
+#endif /* UNTESTED CODE */
+
 guchar*
 gck_data_der_write_certificate (ASN1_TYPE asn1, gsize *n_data)
 {
 	g_return_val_if_fail (asn1, NULL);
 	g_return_val_if_fail (n_data, NULL);
 	
-	return gck_data_asn1_encode (asn1, "", n_data, NULL);
+	return egg_asn1_encode (asn1, "", n_data, NULL);
 }
-
-/* -----------------------------------------------------------------------------
- * CIPHER/KEY DESCRIPTIONS 
- */
- 
-GckDataResult
-gck_data_der_read_cipher (GQuark oid_scheme, const gchar *password, 
-                          const guchar *data, gsize n_data, gcry_cipher_hd_t *cih)
-{
-	GckDataResult ret = GCK_DATA_UNRECOGNIZED;
-	
-	g_return_val_if_fail (oid_scheme != 0, GCK_DATA_FAILURE);
-	g_return_val_if_fail (cih != NULL, GCK_DATA_FAILURE);
-	g_return_val_if_fail (data != NULL && n_data != 0, GCK_DATA_FAILURE);
-	
-	init_quarks ();
-	
-	/* PKCS#5 PBE */
-	if (oid_scheme == OID_PBE_MD2_DES_CBC)
-		ret = gck_data_der_read_cipher_pkcs5_pbe (GCRY_CIPHER_DES, GCRY_CIPHER_MODE_CBC,
-		                                          GCRY_MD_MD2, password, data, n_data, cih);
-
-	else if (oid_scheme == OID_PBE_MD2_RC2_CBC)
-		/* RC2-64 has no implementation in libgcrypt */
-		ret = GCK_DATA_UNRECOGNIZED;
-	else if (oid_scheme == OID_PBE_MD5_DES_CBC)
-		ret = gck_data_der_read_cipher_pkcs5_pbe (GCRY_CIPHER_DES, GCRY_CIPHER_MODE_CBC,
-		                                          GCRY_MD_MD5, password, data, n_data, cih);
-	else if (oid_scheme == OID_PBE_MD5_RC2_CBC)
-		/* RC2-64 has no implementation in libgcrypt */
-		ret = GCK_DATA_UNRECOGNIZED;
-	else if (oid_scheme == OID_PBE_SHA1_DES_CBC)
-		ret = gck_data_der_read_cipher_pkcs5_pbe (GCRY_CIPHER_DES, GCRY_CIPHER_MODE_CBC,
-		                                          GCRY_MD_SHA1, password, data, n_data, cih);
-	else if (oid_scheme == OID_PBE_SHA1_RC2_CBC)
-		/* RC2-64 has no implementation in libgcrypt */
-		ret = GCK_DATA_UNRECOGNIZED;
-
-	
-	/* PKCS#5 PBES2 */
-	else if (oid_scheme == OID_PBES2)
-		ret = gck_data_der_read_cipher_pkcs5_pbes2 (password, data, n_data, cih);
-
-		
-	/* PKCS#12 PBE */
-	else if (oid_scheme == OID_PKCS12_PBE_ARCFOUR_SHA1)
-		ret = gck_data_der_read_cipher_pkcs12_pbe (GCRY_CIPHER_ARCFOUR, GCRY_CIPHER_MODE_STREAM, 
-                                                   password, data, n_data, cih);
-	else if (oid_scheme == OID_PKCS12_PBE_RC4_40_SHA1)
-		/* RC4-40 has no implementation in libgcrypt */;
-
-	else if (oid_scheme == OID_PKCS12_PBE_3DES_SHA1)
-		ret = gck_data_der_read_cipher_pkcs12_pbe (GCRY_CIPHER_3DES, GCRY_CIPHER_MODE_CBC, 
-                                                   password, data, n_data, cih);
-	else if (oid_scheme == OID_PKCS12_PBE_2DES_SHA1) 
-		/* 2DES has no implementation in libgcrypt */;
-		
-	else if (oid_scheme == OID_PKCS12_PBE_RC2_128_SHA1)
-		ret = gck_data_der_read_cipher_pkcs12_pbe (GCRY_CIPHER_RFC2268_128, GCRY_CIPHER_MODE_CBC, 
-                                                   password, data, n_data, cih);
-
-	else if (oid_scheme == OID_PKCS12_PBE_RC2_40_SHA1)
-		ret = gck_data_der_read_cipher_pkcs12_pbe (GCRY_CIPHER_RFC2268_40, GCRY_CIPHER_MODE_CBC, 
-                                                   password, data, n_data, cih);
-
-	if (ret == GCK_DATA_UNRECOGNIZED)
-    		g_message ("unsupported or unrecognized cipher oid: %s", g_quark_to_string (oid_scheme));
-    	return ret;
-}
-
-GckDataResult
-gck_data_der_read_cipher_pkcs5_pbe (int cipher_algo, int cipher_mode, 
-                                    int hash_algo, const gchar *password, const guchar *data, 
-                                    gsize n_data, gcry_cipher_hd_t *cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	gcry_error_t gcry;
-	GckDataResult ret;
-	const guchar *salt;
-	gsize n_salt;
-	gsize n_block, n_key;
-	guint iterations;
-	guchar *key = NULL;
-	guchar *iv = NULL;
-
-	g_return_val_if_fail (cipher_algo != 0 && cipher_mode != 0, GCK_DATA_FAILURE);
-	g_return_val_if_fail (cih != NULL, GCK_DATA_FAILURE);
-	g_return_val_if_fail (data != NULL && n_data != 0, GCK_DATA_FAILURE);
-
-	*cih = NULL;	
-	ret = GCK_DATA_UNRECOGNIZED;
-	
-	/* Check if we can use this algorithm */
-	if (gcry_cipher_algo_info (cipher_algo, GCRYCTL_TEST_ALGO, NULL, 0) != 0 ||
-	    gcry_md_test_algo (hash_algo) != 0)
-		goto done;
-	
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-5-PBE-params", data, n_data);
-	if (!asn) 
-		goto done;
-		
-	ret = GCK_DATA_FAILURE;
-		
-	salt = gck_data_asn1_read_content (asn, data, n_data, "salt", &n_salt);
-	if (!salt)
-		goto done;
-	if (!gck_data_asn1_read_uint (asn, "iterationCount", &iterations))
-		iterations = 1;
-		
-	n_key = gcry_cipher_get_algo_keylen (cipher_algo);
-	g_return_val_if_fail (n_key > 0, GCK_DATA_FAILURE);
-	n_block = gcry_cipher_get_algo_blklen (cipher_algo);
-		
-	if (!gck_crypto_symkey_generate_pbe (cipher_algo, hash_algo, password, salt,
-	                                     n_salt, iterations, &key, n_block > 1 ? &iv : NULL))
-		goto done;
-		
-	gcry = gcry_cipher_open (cih, cipher_algo, cipher_mode, 0);
-	if (gcry != 0) {
-		g_warning ("couldn't create cipher: %s", gcry_strerror (gcry));
-		goto done;
-	}
-	
-	if (iv) 
-		gcry_cipher_setiv (*cih, iv, n_block);
-	gcry_cipher_setkey (*cih, key, n_key);
-	
-	ret = GCK_DATA_SUCCESS;
-
-done:
-	gcry_free (iv);
-	gcry_free (key);
-	
-	if (asn)
-		asn1_delete_structure (&asn);
-		
-	return ret;
-}
-
-static gboolean
-setup_pkcs5_rc2_params (const guchar *data, guchar n_data, gcry_cipher_hd_t cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	gcry_error_t gcry;
-	const guchar *iv;
-	gsize n_iv;
-	guint version;
-	
-	g_assert (data);
-
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-5-rc2-CBC-params", data, n_data);
-	if (!asn) 
-		return GCK_DATA_UNRECOGNIZED;
-		
-	if (!gck_data_asn1_read_uint (asn, "rc2ParameterVersion", &version))
-		return GCK_DATA_FAILURE;
-	
-	iv = gck_data_asn1_read_content (asn, data, n_data, "iv", &n_iv);
-	asn1_delete_structure (&asn);
-
-	if (!iv)
-		return GCK_DATA_FAILURE;
-		
-	gcry = gcry_cipher_setiv (cih, iv, n_iv);
-			
-	if (gcry != 0) {
-		g_message ("couldn't set %lu byte iv on cipher", (gulong)n_iv);
-		return GCK_DATA_FAILURE;
-	}
-	
-	return GCK_DATA_SUCCESS;
-}
-
-static gboolean
-setup_pkcs5_des_params (const guchar *data, guchar n_data, gcry_cipher_hd_t cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	gcry_error_t gcry;
-	const guchar *iv;
-	gsize n_iv;
-	
-	g_assert (data);
-
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-5-des-EDE3-CBC-params", data, n_data);
-	if (!asn)
-		asn = gck_data_asn1_decode ("PKIX1.pkcs-5-des-CBC-params", data, n_data);
-	if (!asn) 
-		return GCK_DATA_UNRECOGNIZED;
-	
-	iv = gck_data_asn1_read_content (asn, data, n_data, "", &n_iv);
-	asn1_delete_structure (&asn);
-
-	if (!iv)
-		return GCK_DATA_FAILURE;
-		
-	gcry = gcry_cipher_setiv (cih, iv, n_iv);
-			
-	if (gcry != 0) {
-		g_message ("couldn't set %lu byte iv on cipher", (gulong)n_iv);
-		return GCK_DATA_FAILURE;
-	}
-	
-	return GCK_DATA_SUCCESS;
-}
-
-static GckDataResult
-setup_pkcs5_pbkdf2_params (const gchar *password, const guchar *data, 
-                           gsize n_data, int cipher_algo, gcry_cipher_hd_t cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	GckDataResult ret;
-	gcry_error_t gcry;
-	guchar *key = NULL; 
-	const guchar *salt;
-	gsize n_salt, n_key;
-	guint iterations;
-	
-	g_assert (cipher_algo);
-	g_assert (data);
-	
-	ret = GCK_DATA_UNRECOGNIZED;
-
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-5-PBKDF2-params", data, n_data);
-	if (!asn)
-		goto done;
-		
-	ret = GCK_DATA_FAILURE;
-		
-	if (!gck_data_asn1_read_uint (asn, "iterationCount", &iterations))
-		iterations = 1;
-	salt = gck_data_asn1_read_content (asn, data, n_data, "salt.specified", &n_salt);
-	if (!salt)
-		goto done;
-				
-	if (!gck_crypto_symkey_generate_pbkdf2 (cipher_algo, GCRY_MD_SHA1, password, 
-	                                        salt, n_salt, iterations, &key, NULL))
-		goto done;
-
-	n_key = gcry_cipher_get_algo_keylen (cipher_algo);
-	g_return_val_if_fail (n_key > 0, GCK_DATA_FAILURE);
-	
-	gcry = gcry_cipher_setkey (cih, key, n_key);
-	if (gcry != 0) {
-		g_message ("couldn't set %lu byte key on cipher", (gulong)n_key);
-		goto done;
-	}
-	
-	ret = GCK_DATA_SUCCESS;
-	                                         
-done:
-	gcry_free (key);
-	if (asn)
-		asn1_delete_structure (&asn);
-	return ret;
-}
-
-GckDataResult
-gck_data_der_read_cipher_pkcs5_pbes2 (const gchar *password, const guchar *data, 
-                                      gsize n_data, gcry_cipher_hd_t *cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	GckDataResult r, ret;
-	GQuark key_deriv_algo, enc_oid;
-	gcry_error_t gcry;
-	int algo, mode;
-	int beg, end, res;
-
-	g_return_val_if_fail (cih != NULL, GCK_DATA_FAILURE);
-	g_return_val_if_fail (data != NULL && n_data != 0, GCK_DATA_FAILURE);
-	
-	init_quarks ();
-	
-	*cih = NULL;
-	ret = GCK_DATA_UNRECOGNIZED;
-	
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-5-PBES2-params", data, n_data);
-	if (!asn)
-		goto done;
-		
-	res = GCK_DATA_FAILURE;
-	algo = mode = 0;
-	
-	/* Read in all the encryption type */
-	enc_oid = gck_data_asn1_read_oid (asn, "encryptionScheme.algorithm");
-	if (!enc_oid)
-		goto done;	
-	if (enc_oid == OID_DES_EDE3_CBC)
-		algo = GCRY_CIPHER_3DES;
-	else if (enc_oid == OID_DES_CBC)
-		algo = GCRY_CIPHER_DES;
-	else if (enc_oid == OID_DES_RC2_CBC)
-		algo = GCRY_CIPHER_RFC2268_128;
-	else if (enc_oid == OID_DES_RC5_CBC)
-		/* RC5 doesn't exist in libgcrypt */;
-	
-	/* Unsupported? */
-	if (algo == 0 || gcry_cipher_algo_info (algo, GCRYCTL_TEST_ALGO, NULL, 0) != 0) {
-		ret = GCK_DATA_UNRECOGNIZED;
-		goto done;
-	}
-
-	/* Instantiate our cipher */
-	gcry = gcry_cipher_open (cih, algo, GCRY_CIPHER_MODE_CBC, 0);
-	if (gcry != 0) {
-		g_warning ("couldn't create cipher: %s", gcry_cipher_algo_name (algo));
-		goto done;
-	}
-		
-	/* Read out the parameters */
-	if (asn1_der_decoding_startEnd (asn, data, n_data, "encryptionScheme.parameters",
-	                                &beg, &end) != ASN1_SUCCESS)
-		goto done;
-		
-	switch (algo) {
-	case GCRY_CIPHER_3DES:
-	case GCRY_CIPHER_DES:
-		r = setup_pkcs5_des_params (data + beg, end - beg + 1, *cih);
-		break;
-	case GCRY_CIPHER_RFC2268_128:
-		r = setup_pkcs5_rc2_params (data + beg, end - beg + 1, *cih);
-		break;
-	default:
-		/* Should have been caught on the oid check above */
-		g_assert_not_reached ();
-		r = GCK_DATA_UNRECOGNIZED;
-		break;
-	};
-
-	if (r != GCK_DATA_SUCCESS) {
-		ret = r;
-		goto done;
-	}
-
-	/* Read out the key creation paramaters */
-	key_deriv_algo = gck_data_asn1_read_oid (asn, "keyDerivationFunc.algorithm");
-	if (!key_deriv_algo)
-		goto done;
-	if (key_deriv_algo != OID_PBKDF2) {
-		g_message ("unsupported key derivation algorithm: %s", g_quark_to_string (key_deriv_algo));
-		ret = GCK_DATA_UNRECOGNIZED;
-		goto done;
-	}
-
-	if (asn1_der_decoding_startEnd (asn, data, n_data, "keyDerivationFunc.parameters",
-	                                &beg, &end) != ASN1_SUCCESS)
-		goto done;
-	
-	ret = setup_pkcs5_pbkdf2_params (password, data + beg, end - beg + 1, algo, *cih);
-
-done:
-	if (ret != GCK_DATA_SUCCESS && *cih) {
-		gcry_cipher_close (*cih);
-		*cih = NULL;
-	}
-	
-	if (asn)
-		asn1_delete_structure (&asn);
-	
-	return ret;
-}
-
-GckDataResult
-gck_data_der_read_cipher_pkcs12_pbe (int cipher_algo, int cipher_mode, const gchar *password, 
-                                     const guchar *data, gsize n_data, gcry_cipher_hd_t *cih)
-{
-	ASN1_TYPE asn = ASN1_TYPE_EMPTY;
-	gcry_error_t gcry;
-	GckDataResult ret;
-	const guchar *salt;
-	gsize n_salt;
-	gsize n_block, n_key;
-	guint iterations;
-	guchar *key = NULL;
-	guchar *iv = NULL;
-	
-	g_return_val_if_fail (cipher_algo != 0 && cipher_mode != 0, GCK_DATA_FAILURE);
-	g_return_val_if_fail (cih != NULL, GCK_DATA_FAILURE);
-	g_return_val_if_fail (data != NULL && n_data != 0, GCK_DATA_FAILURE);
-	
-	*cih = NULL;
-	ret = GCK_DATA_UNRECOGNIZED;
-	
-	/* Check if we can use this algorithm */
-	if (gcry_cipher_algo_info (cipher_algo, GCRYCTL_TEST_ALGO, NULL, 0) != 0)
-		goto done;
-	
-	asn = gck_data_asn1_decode ("PKIX1.pkcs-12-PbeParams", data, n_data);
-	if (!asn)
-		goto done;
-
-	ret = GCK_DATA_FAILURE;
-
-	salt = gck_data_asn1_read_content (asn, data, n_data, "salt", &n_salt);
-	if (!salt)
-		goto done;
-	if (!gck_data_asn1_read_uint (asn, "iterations", &iterations))
-		goto done;
-	
-	n_block = gcry_cipher_get_algo_blklen (cipher_algo);
-	n_key = gcry_cipher_get_algo_keylen (cipher_algo);
-	
-	/* Generate IV and key using salt read above */
-	if (!gck_crypto_symkey_generate_pkcs12 (cipher_algo, GCRY_MD_SHA1, password,
-	                                        salt, n_salt, iterations, &key, 
-	                                        n_block > 1 ? &iv : NULL))
-		goto done;
-		
-	gcry = gcry_cipher_open (cih, cipher_algo, cipher_mode, 0);
-	if (gcry != 0) {
-		g_warning ("couldn't create encryption cipher: %s", gcry_strerror (gcry));
-		goto done;
-	}
-	
-	if (iv) 
-		gcry_cipher_setiv (*cih, iv, n_block);
-	gcry_cipher_setkey (*cih, key, n_key);
-	
-	ret = GCK_DATA_SUCCESS;
-	
-done:
-	if (ret != GCK_DATA_SUCCESS && *cih) {
-		gcry_cipher_close (*cih);
-		*cih = NULL;
-	}
-	
-	gcry_free (iv);
-	gcry_free (key);
-	
-	if (asn)
-		asn1_delete_structure (&asn);
-	
-	return ret;
-}
-
-#endif /* UNTESTED_CODE */
