@@ -33,22 +33,22 @@
 
 #include "gtest-helpers.h"
 
-#include "common/gkr-secure-memory.h"
+#include "egg/egg-secure-memory.h"
 
 static GStaticMutex memory_mutex = G_STATIC_MUTEX_INIT;
 static const gchar *test_path = NULL;
 
-void gkr_memory_lock (void) 
+void egg_memory_lock (void) 
 { 
 	g_static_mutex_lock (&memory_mutex); 
 }
 
-void gkr_memory_unlock (void) 
+void egg_memory_unlock (void) 
 { 
 	g_static_mutex_unlock (&memory_mutex); 
 }
 
-void* gkr_memory_fallback (void *p, unsigned long sz) 
+void* egg_memory_fallback (void *p, unsigned long sz) 
 { 
 	return g_realloc (p, sz); 
 }
@@ -94,6 +94,16 @@ test_build_filename (const gchar *basename)
 	return g_build_filename (test_path, basename, NULL);
 }
 
+guchar* 
+test_read_testdata (const gchar *basename, gsize *n_result)
+{
+	gchar *file = g_build_filename ("test-data", basename, NULL);
+	gchar *result;
+	if (!g_file_get_contents (file, &result, n_result, NULL))
+		g_assert_not_reached ();
+	return (guchar*)result;
+}
+
 static void 
 chdir_base_dir (char* argv0)
 {
